/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MainForm.cpp,v 1.53 2003/09/17 13:06:05 comix Exp $
 *
 */


#include "MainForm.h"

//----------------------------------------------------------------------------
/**
 * Constructor
 */
MainForm::MainForm(QApplication *app, string songFilename ) : QMainWindow( 0, 0, 0 ) {
//	cout << "MainForm init" << endl;
	string icon_path = string( IMG_PATH ) + string( "/img/icon32.png" );
	setIcon( QPixmap( icon_path.c_str() ) );

	m_pQApp = app;

	workspace = NULL;

	m_pQApp->processEvents( 1000 );

	// Load default song
	Song *song = NULL;
	if (songFilename != "") {
		song = Song::load( songFilename );
		if (song == NULL) {
			QMessageBox::warning( this, "Hydrogen", trUtf8("Error loading song.") );
			song = Song::getEmptySong();
		}
	}
	else {
		PreferencesMng *pref = PreferencesMng::getInstance();
		bool restoreLastSong = pref->isRestoreLastSongEnabled();
		string filename = pref->getLastSongFilename();
		if ( restoreLastSong && (filename != "" )) {
			song = Song::load( filename );
			if (song == NULL) {
				QMessageBox::warning( this, "Hydrogen", trUtf8("Error restoring last song.") );
				song = Song::getEmptySong();
			}
		}
		else {
			song = Song::getEmptySong();
		}
	}

	h2app = new HydrogenApp( this, song );
//	h2app->setSong( song );


	createMenuBar();

	// create toolbar with player control
	QToolBar * fileTools = new QToolBar( this, "file operations" );
        fileTools->setMovingEnabled( false );
	fileTools->setLabel( "Hydrogen control panel" );
	PlayerControl *playerControl;
	playerControl = new PlayerControl( fileTools );

	h2app->setStatusBarMessage( (trUtf8("Hydrogen Ready.")).latin1(), 10000 );
}





//----------------------------------------------------------------------------
/**
 * Destructor
 */
MainForm::~MainForm() {
	if ( ((Hydrogen::getInstance())->getState() == PLAYING) ) {
		(Hydrogen::getInstance())->stop();
	}

	hide();

	if (h2app != NULL) {
		delete h2app;
		h2app = NULL;
	}
}




//----------------------------------------------------------------------------
/**
 * Create the menubar
 */
void MainForm::createMenuBar() {
	// actions
	menuItem_file_open = new QAction( this, "menuItem_file_open" );
	menuItem_file_open->setText( trUtf8( "Open" ) );

	menuItem_file_openDemo = new QAction( this, "menuItem_file_openDemo" );
	menuItem_file_openDemo->setText( trUtf8( "Open Demo" ) );

	menuItem_file_exit = new QAction( this, "menuItem_file_exit" );
	menuItem_file_exit->setText( trUtf8( "Exit" ) );
	menuItem_file_new = new QAction( this, "menuItem_file_new" );
	menuItem_file_new->setText( trUtf8( "New" ) );
	menuItem_file_save_as = new QAction( this, "menuItem_file_save_as" );
	menuItem_file_save_as->setText( trUtf8( "Save as..." ) );
	menuItem_file_export = new QAction( this, "menuItem_file_export" );
	menuItem_file_export->setText( trUtf8( "Export" ) );
	menuItem_file_preferences = new QAction( this, "menuItem_file_preferences" );
	menuItem_file_preferences->setText( trUtf8( "Preferences" ) );

	menuItem_window_showMixer = new QAction( this, "menuItem_window_showMixer" );
	menuItem_window_showMixer->setText( trUtf8( "Show mixer" ) );

	menuItem_window_showAudioEngineInfo = new QAction( this, "menuItem_window_showAudioEngineInfo" );
	menuItem_window_showAudioEngineInfo->setText( trUtf8( "Show audio engine info" ) );

	menuItem_window_showSongEditor = new QAction( this, "menuItem_window_showSongEditor" );
	menuItem_window_showSongEditor->setText( trUtf8( "Show song editor" ) );

	menuItem_window_showDrumkitManager = new QAction( this, "menuItem_window_showDrumkitManager" );
	menuItem_window_showDrumkitManager->setText( trUtf8( "Show drumkit manager" ) );

	menuItem_help_manual = new QAction( this, "menuItem_help_manual" );
	menuItem_help_manual->setText( trUtf8( "User manual" ) );

	menuItem_help_about = new QAction( this, "menuItem_help_about" );
	menuItem_help_about->setText( trUtf8( "About" ) );

	// menubar
	menubar = new QMenuBar( this, "menubar" );

	filePopupMenu = new QPopupMenu( this );
	menuItem_file_new->addTo( filePopupMenu );
	filePopupMenu->insertSeparator();
	menuItem_file_open->addTo( filePopupMenu );
	menuItem_file_openDemo->addTo( filePopupMenu );
	filePopupMenu->insertSeparator();
	menuItem_file_save_as->addTo( filePopupMenu );
	menuItem_file_export->addTo( filePopupMenu );
	filePopupMenu->insertSeparator();
	menuItem_file_preferences->addTo( filePopupMenu );
	filePopupMenu->insertSeparator();

	menuItem_file_exit->addTo( filePopupMenu);
	menubar->insertItem( trUtf8("&File"), filePopupMenu );

	windowPopupMenu = new QPopupMenu( this );
	menubar->insertItem( trUtf8( "&Window" ), windowPopupMenu );
	menuItem_window_showMixer->addTo( windowPopupMenu );
	if ( ( PreferencesMng::getInstance() )->getInterfaceMode() == "Child frame" ) {	// show only with child frame interface
		menuItem_window_showSongEditor->addTo( windowPopupMenu );
	}
	menuItem_window_showDrumkitManager->addTo( windowPopupMenu );
	windowPopupMenu->insertSeparator();
	menuItem_window_showAudioEngineInfo->addTo( windowPopupMenu );

	helpPopupMenu = new QPopupMenu( this );
	menuItem_help_manual->addTo( helpPopupMenu );
	helpPopupMenu->insertSeparator();
	menuItem_help_about->addTo( helpPopupMenu );
	menubar->insertItem( trUtf8( "?" ), helpPopupMenu );

	// signals and slots connections
	connect( menuItem_file_open, SIGNAL( activated() ), this, SLOT( action_file_open() ) );
	connect( menuItem_file_openDemo, SIGNAL( activated() ), this, SLOT( action_file_openDemo() ) );
	connect( menuItem_file_save_as, SIGNAL( activated() ), this, SLOT( action_file_save_as() ) );
	connect( menuItem_file_new, SIGNAL( activated() ), this, SLOT( action_file_new() ) );
	connect( menuItem_file_exit, SIGNAL( activated() ), this, SLOT( action_file_exit() ) );
	connect( menuItem_file_preferences, SIGNAL( activated() ), this, SLOT( action_file_preferences() ) );
	connect( menuItem_file_export, SIGNAL( activated() ), this, SLOT( action_file_export() ) );
	connect( menuItem_window_showMixer, SIGNAL( activated() ), this, SLOT( action_window_showMixer() ) );
	connect( menuItem_window_showAudioEngineInfo, SIGNAL( activated() ), this, SLOT( action_window_showAudioEngineInfo() ) );
	connect( menuItem_window_showSongEditor, SIGNAL( activated() ), this, SLOT( action_window_showSongEditor() ) );
	connect( menuItem_window_showDrumkitManager, SIGNAL( activated() ), this, SLOT( action_window_showDrumkitManager() ) );
	connect( menuItem_help_manual, SIGNAL( activated() ), this, SLOT( action_help_manual() ) );
	connect( menuItem_help_about, SIGNAL( activated() ), this, SLOT( action_help_about() ) );

}





//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_file_exit() {
	if ( (h2app->getSong())->isModified() ) {
		switch(
				QMessageBox::information( this, "Hydrogen",
						trUtf8("\nThe song has unsaved changes\n"
						"Do you want to save the changes before exiting?\n"),
						trUtf8("&Save"), trUtf8("&Discard"), trUtf8("Cancel"),
						0,      // Enter == button 0
						2 ) ) { // Escape == button 2
			case 0:
				// Save clicked or Alt+S pressed or Enter pressed.
				action_file_save_as();
				// save
				break;
			case 1:
				// Discard clicked or Alt+D pressed
				// don't save but exit
				break;
			case 2:
				// Cancel clicked or Alt+C pressed or Escape pressed
				// don't exit
				return;
				break;
		}
	}


//	(Hydrogen::getInstance())->stop();

	closeAll();
//	this->app->quit();
}





//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_file_new() {
	if ( ((Hydrogen::getInstance())->getState() == PLAYING) ) {
		(Hydrogen::getInstance())->stop();
	}

	if ( (h2app->getSong())->isModified() ) {
		switch(
				QMessageBox::information( this, "Hydrogen",
						trUtf8("\nThe document contains unsaved changes\n"
						"Do you want to save the changes before exiting?\n"),
						trUtf8("&Save"), trUtf8("&Discard"), trUtf8("Cancel"),
						0,      // Enter == button 0
						2 ) ) { // Escape == button 2
			case 0: // Save clicked or Alt+S pressed or Enter pressed.
				action_file_save_as();
				// save
				break;
			case 1: // Discard clicked or Alt+D pressed
				// don't save but exit
				break;
			case 2: // Cancel clicked or Alt+C pressed or Escape pressed
				// don't exit
				return;
				break;
		}
	}

	h2app->setSong(Song::getEmptySong());
}




//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_file_save_as() {
	if ( ((Hydrogen::getInstance())->getState() == PLAYING) ) {
		(Hydrogen::getInstance())->stop();
	}

	QFileDialog *fd = new QFileDialog(this, "File Dialog", TRUE);
	fd->setMode(QFileDialog::AnyFile);
	fd->setFilter( trUtf8("Hydrogen Song (*.h2song)") );

	fd->setCaption( trUtf8( "Hydrogen - Save song" ) );
	string icon_path = IMG_PATH;
	icon_path.append("/img/icon32.png");
	fd->setIcon(QPixmap(icon_path.c_str()));

	Song *song = h2app->getSong();
	QString defaultFilename;
	QString lastFilename = (song->getFilename()).c_str();

	if (lastFilename == "") {
		defaultFilename = (Hydrogen::getInstance()->getSong()->getName()).c_str();
		defaultFilename += ".h2song";
	} else {
		defaultFilename = lastFilename;
	}

	fd->setSelection(defaultFilename);

	QString filename = "";
	if (fd->exec() == QDialog::Accepted) {
		filename = fd->selectedFile();
	}

	if (filename != "") {
		LocalFileMng mng;
		song->save( filename.latin1() );
		( PreferencesMng::getInstance() )->setLastSongFilename( song->getFilename() );
	}
}





//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_help_about() {
	QWidget *parent = this;
	if (workspace) {
		parent = workspace;
	}

	// show modal dialog
	AboutDialog *dialog = new AboutDialog( NULL );
	dialog->exec();
}






//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_help_manual() {
	QWidget *parent = this;
	if (workspace) {
		parent = workspace;
	}

	h2app->getHelpBrowser()->hide();
	h2app->getHelpBrowser()->show();
}





//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_file_open() {
	if ( ((Hydrogen::getInstance())->getState() == PLAYING) ) {
		(Hydrogen::getInstance())->stop();
	}

	if ( (h2app->getSong())->isModified() ) {
		switch(
			QMessageBox::information( this, "Hydrogen",
					trUtf8("\nThe document contains unsaved changes\n"
					"Do you want to save the changes before exiting?\n"),
					trUtf8("&Save"), trUtf8("&Discard"), trUtf8("Cancel"),
					0,      // Enter == button 0
					2 ) ) { // Escape == button 2
			case 0: // Save clicked or Alt+S pressed or Enter pressed.
				action_file_save_as();
				// save
				break;
			case 1: // Discard clicked or Alt+D pressed
				// don't save but exit
				break;
			case 2: // Cancel clicked or Alt+C pressed or Escape pressed
				// don't exit
				return;
				break;
		}
	}

	static QString lastUsedDir = "";

	QFileDialog *fd = new QFileDialog(this, "File Dialog", TRUE);
	fd->setMode(QFileDialog::ExistingFile);
	fd->setFilter( trUtf8("Hydrogen Song (*.h2song)") );
	fd->setDir( lastUsedDir );

	fd->setCaption( trUtf8( "Hydrogen - Open song" ) );
	string icon_path = string( IMG_PATH ) + string( "/img/icon32.png" );
	fd->setIcon( QPixmap( icon_path.c_str() ) );

	// TODO: impostare il preview
	/*
	fd->setContentsPreviewEnabled( TRUE );
	fd->setContentsPreview( "uno", "due" );
	fd->setPreviewMode( QFileDialog::Contents );
	*/

	QString filename = "";
	if (fd->exec() == QDialog::Accepted) {
		filename = fd->selectedFile();
		lastUsedDir = fd->dirPath();
	}


	if (filename != "") {
		LocalFileMng mng;
		Song *song = mng.loadSong(filename.latin1());
		if (song == NULL) {
			QMessageBox::information( this, "Hydrogen", trUtf8("Error loading song.") );
			return;
		}

		h2app->setSong( song );
	}
}




// TODO: parametrizzare il metodo action_file_open ed eliminare il seguente...
//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_file_openDemo() {
	if ( ((Hydrogen::getInstance())->getState() == PLAYING) ) {
		(Hydrogen::getInstance())->stop();
	}

	if ( (h2app->getSong())->isModified() ) {
		switch(
			QMessageBox::information( this, "Hydrogen",
					trUtf8("\nThe document contains unsaved changes\n"
					"Do you want to save the changes before exiting?\n"),
					trUtf8("&Save"), trUtf8("&Discard"), trUtf8("Cancel"),
					0,      // Enter == button 0
					2 ) ) { // Escape == button 2
			case 0: // Save clicked or Alt+S pressed or Enter pressed.
				action_file_save_as();
				// save
				break;
			case 1: // Discard clicked or Alt+D pressed
				// don't save but exit
				break;
			case 2: // Cancel clicked or Alt+C pressed or Escape pressed
				// don't exit
				return;
				break;
		}
	}


	QFileDialog *fd = new QFileDialog(this, "File Dialog", TRUE);
	fd->setMode(QFileDialog::ExistingFile);
	fd->setFilter( trUtf8("Hydrogen Song (*.h2song)") );

	fd->setCaption( trUtf8( "Hydrogen - Open song" ) );
	string icon_path = IMG_PATH;
	icon_path.append("/img/icon32.png");
	fd->setIcon(QPixmap(icon_path.c_str()));

	// TODO: impostare il preview
	/*
	fd->setContentsPreviewEnabled( TRUE );
	fd->setContentsPreview( "uno", "due" );
	fd->setPreviewMode( QFileDialog::Contents );
	*/
	string demoPath = (PreferencesMng::getInstance())->getDemoPath();
	fd->setSelection( QString( demoPath.c_str() ) );


	QString filename = "";
	if (fd->exec() == QDialog::Accepted) {
		filename = fd->selectedFile();
	}


	if (filename != "") {
		LocalFileMng mng;
		Song *song = mng.loadSong(filename.latin1());
		if (song == NULL) {
			QMessageBox::information( this, "Hydrogen", trUtf8("Error loading song.") );
			return;
		}

		song->setFilename("");

		h2app->setSong( song );
	}
}





//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_file_preferences() {
	if ( ((Hydrogen::getInstance())->getState() == PLAYING) ) {
		(Hydrogen::getInstance())->stop();
	}

	h2app->showPreferencesDialog();
}






//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_window_showMixer() {
	h2app->showMixer(true);
}





//----------------------------------------------------------------------------
/**
 *
 */
void MainForm::action_window_showAudioEngineInfo() {
	h2app->showAudioEngineInfoForm();
}





//----------------------------------------------------------------------------
/**
 * Show the song editor
 */
void MainForm::action_window_showSongEditor() {
	h2app->getSongEditorPanel()->hide();
	h2app->getSongEditorPanel()->show();
}




//----------------------------------------------------------------------------
/**
 * Window close event
 */
void MainForm::closeEvent( QCloseEvent* ev ) {
	if ( (h2app->getSong())->isModified() ) {
		switch(
			QMessageBox::information( this, "Hydrogen",
					trUtf8("\nThe song has unsaved changes\n"
					"Do you want to save the changes before exiting?\n"),
					trUtf8("&Save"), trUtf8("&Discard"), trUtf8("Cancel"),
					0,      // Enter == button 0
					2 ) ) { // Escape == button 2
			case 0:
				// Save clicked or Alt+S pressed or Enter pressed.
				action_file_save_as();
				// save
				break;
			case 1:
				// Discard clicked or Alt+D pressed
				// don't save but exit
				break;
			case 2:
				// Cancel clicked or Alt+C pressed or Escape pressed
				// don't exit
				return;
				break;
		}
	}


	if ( ((Hydrogen::getInstance())->getState() == PLAYING) ) {
		(Hydrogen::getInstance())->stop();
	}


	closeAll();
//	this->app->quit();
	ev->accept();
}




/**
 *
 */
void MainForm::action_file_export() {
	ExportSongDialog *dialog = new ExportSongDialog(this);
	dialog->exec();
	delete dialog;
}




/**
 *
 */
void MainForm::action_window_showDrumkitManager() {
	h2app->getDrumkitManager()->hide();
	h2app->getDrumkitManager()->show();
}




/**
 *
 */
void MainForm::customEvent( QCustomEvent *ev ) {
	if ( ev->type() != H2_TEXT_EVENT ) {	// Must be a H2TextEvent
		return;
	}
	string message = ((H2TextEvent*)ev)->getText();

	if (message == "ErrorMsg" ) {
		string msg = ((H2TextEvent*)ev)->getErrorMsg();
		QMessageBox::information( this, "Hydrogen", QString( msg.c_str() ) );
	}

}




/**
 *
 */
void MainForm::closeAll() {

	// save window properties in the preferences files
	PreferencesMng *pref = PreferencesMng::getInstance();

	// mainform
	WindowProperties mainFormProp;
	mainFormProp.x = x();
	mainFormProp.y = y();
	mainFormProp.height = height();
	mainFormProp.width = width();
	pref->setMainFormProperties( mainFormProp );

	// Save mixer properties
	WindowProperties mixerProp;
	if ( h2app->getMixer()->parentWidget() != 0 ) {	// use the new MDI interface
		mixerProp.x = h2app->getMixer()->parentWidget()->x();
		mixerProp.y = h2app->getMixer()->parentWidget()->y();
	}
	else {
		mixerProp.x = h2app->getMixer()->x();
		mixerProp.y = h2app->getMixer()->y();
	}
	pref->setMixerProperties( mixerProp );

	// save pattern editor properties
	WindowProperties patternEditorProp;
	if ( h2app->getPatternEditor()->parentWidget() != 0 ) {	// use the new MDI interface
		patternEditorProp.x = h2app->getPatternEditor()->parentWidget()->x();
		patternEditorProp.y = h2app->getPatternEditor()->parentWidget()->y();
	}
	else {
		patternEditorProp.x = h2app->getPatternEditor()->x();
		patternEditorProp.y = h2app->getPatternEditor()->y();
	}
	pref->setPatternEditorProperties( patternEditorProp );

	// save song editor properties
	WindowProperties songEditorProp;
	if ( h2app->getSongEditorPanel()->parentWidget() != 0 ) {	// use the new MDI interface
		songEditorProp.x = h2app->getSongEditorPanel()->parentWidget()->x();
		songEditorProp.y = h2app->getSongEditorPanel()->parentWidget()->y();
	}
	else {
		songEditorProp.x = h2app->getSongEditorPanel()->x();
		songEditorProp.y = h2app->getSongEditorPanel()->y();
	}
	pref->setSongEditorProperties( songEditorProp );

	// save drumkit manager properties
	WindowProperties drumkitMngProp;
	if ( h2app->getDrumkitManager()->parentWidget() != 0 ) {	// use the new MDI interface
		drumkitMngProp.x = h2app->getDrumkitManager()->parentWidget()->x();
		drumkitMngProp.y = h2app->getDrumkitManager()->parentWidget()->y();
	}
	else {
		drumkitMngProp.x = h2app->getDrumkitManager()->x();
		drumkitMngProp.y = h2app->getDrumkitManager()->y();
	}
	pref->setDrumkitManagerProperties( drumkitMngProp );

	// save audio engine info properties
	WindowProperties audioEngineInfoProp;
	if ( h2app->getAudioEngineInfoForm()->parentWidget() != 0 ) {	// use the new MDI interface
		audioEngineInfoProp.x = h2app->getAudioEngineInfoForm()->parentWidget()->x();
		audioEngineInfoProp.y = h2app->getAudioEngineInfoForm()->parentWidget()->y();
	}
	else {
		audioEngineInfoProp.x = h2app->getAudioEngineInfoForm()->x();
		audioEngineInfoProp.y = h2app->getAudioEngineInfoForm()->y();
	}
	pref->setAudioEngineInfoProperties( audioEngineInfoProp );

	m_pQApp->quit();
}


