/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MainForm.h,v 1.26 2003/09/17 13:06:05 comix Exp $
 *
 */


#ifndef MAINFORM_H
#define MAINFORM_H


#include "qapplication.h"
#include "qmainwindow.h"
#include "qaction.h"
#include "qmenubar.h"
#include "qmessagebox.h"
#include "qfiledialog.h"
#include "qstatusbar.h"
#include "qtoolbutton.h"
#include "qworkspace.h"
#include "qvbox.h"

#include "AboutDialog.h"
#include "HydrogenApp.h"
#include "PlayerControl.h"
#include "ExportSongDialog.h"
#include "HelpBrowser.h"
#include "DrumkitManager.h"

class HydrogenApp;

/**
 * Main window
 */
class MainForm : public QMainWindow {
	Q_OBJECT

	public:
		QWorkspace *workspace;

		/** Constructor */
		MainForm( QApplication *app,  string songFilename );

		/** Destructor */
		~MainForm();

		QApplication* m_pQApp;

	public slots:
		void action_file_new();
		void action_file_open();
		void action_file_openDemo();
		void action_file_save_as();
		void action_file_exit();
		void action_file_preferences();
		void action_file_export();
		void action_help_about();
		void action_help_manual();
		void action_window_showMixer();
		void action_window_showAudioEngineInfo();
		void action_window_showSongEditor();
		void action_window_showDrumkitManager();

		void closeEvent( QCloseEvent* ev );

	private:
		static const int topLevel_width = 250;
		static const int topLevel_height= 400;

		HydrogenApp* h2app;


		QStatusBar *statusBar;

		QMenuBar *menubar;
		QPopupMenu *filePopupMenu;
		QPopupMenu *helpPopupMenu;
		QPopupMenu *windowPopupMenu;

		QAction* menuItem_file_open;
		QAction* menuItem_file_openDemo;
		QAction* menuItem_file_exit;
		QAction* menuItem_file_new;
		QAction* menuItem_file_save_as;
		QAction* menuItem_file_export;
		QAction* menuItem_file_preferences;

		QAction* menuItem_help_about;
		QAction* menuItem_help_manual;

		QAction* menuItem_window_showSongEditor;
		QAction* menuItem_window_showMixer;
		QAction* menuItem_window_showDrumkitManager;
		QAction* menuItem_window_showAudioEngineInfo;

		/** Create the menubar */
		void createMenuBar();
		void customEvent( QCustomEvent *ev );

		void closeAll();
};

#endif

