/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MixerLine.cpp,v 1.49 2003/09/25 16:17:28 comix Exp $
 *
 */


#include "MixerLine.h"
#include <stdio.h>

/**
 * Constructor
 */
MixerLine::MixerLine(QWidget* parent) : QWidget(parent) {
//	cout << "mixerLine init" << endl;

	setMinimumSize( width, height );
	setMaximumSize( width, height );
	resize( width, height );

	maxPeak = 0.0;
	activity = 0;

	// MixerLine Background image
	string mixerLineBackground_path = string(IMG_PATH).append("/img/mixerPanel/mixerline_background.png");
	bool ok = mixerLineBackground.load(mixerLineBackground_path.c_str());
	if( ok == false ){
		qWarning( "MixerLine: Error loading pixmap" );
	}
	setBackgroundPixmap(mixerLineBackground);


	// Play sample button
	string playSample_on_path = string(IMG_PATH).append("/img/mixerPanel/btn_play_on.png");
	string playSample_off_path = string(IMG_PATH).append("/img/mixerPanel/btn_play_off.png");
	string playSample_over_path = string(IMG_PATH).append("/img/mixerPanel/btn_play_over.png");
	playSampleBtn = new Button(this, QSize( 18, 12 ), playSample_on_path, playSample_off_path, playSample_over_path);
	playSampleBtn->move( 2, 5 );
	QToolTip::add( playSampleBtn, trUtf8( "Play sample" ) );
	connect(playSampleBtn, SIGNAL(clicked(Button*)), this, SLOT(click(Button*)));
	connect(playSampleBtn, SIGNAL(rightClicked(Button*)), this, SLOT(rightClick(Button*)));

	// load button
	string load_off_path = string(IMG_PATH).append("/img/mixerPanel/btn_load_off.png");
	string load_over_path = string(IMG_PATH).append("/img/mixerPanel/btn_load_over.png");
	loadBtn = new Button(this, QSize( 18, 12 ), load_over_path, load_off_path, load_over_path);
	loadBtn->move( 22, 5 );
	QToolTip::add( loadBtn, trUtf8( "Load external sample" ) );
	connect(loadBtn, SIGNAL(clicked(Button*)), this, SLOT(click(Button*)));

	// Mute button
	string mute_on_path = string(IMG_PATH).append("/img/mixerPanel/btn_mute_on.png");
	string mute_off_path = string(IMG_PATH).append("/img/mixerPanel/btn_mute_off.png");
	string mute_over_path = string(IMG_PATH).append("/img/mixerPanel/btn_mute_over.png");
	muteBtn = new ToggleButton(this, QSize( 18, 12 ), mute_on_path, mute_off_path, mute_over_path);
	muteBtn->move( 2, 20 );
	QToolTip::add( muteBtn, trUtf8( "Mute" ) );
	connect(muteBtn, SIGNAL(clicked(Button*)), this, SLOT(click(Button*)));

	// Solo button
	string solo_on_path = string(IMG_PATH).append("/img/mixerPanel/btn_solo_on.png");
	string solo_off_path = string(IMG_PATH).append("/img/mixerPanel/btn_solo_off.png");
	string solo_over_path = string(IMG_PATH).append("/img/mixerPanel/btn_solo_over.png");
	soloBtn = new ToggleButton(this, QSize( 18, 12 ), solo_on_path, solo_off_path, solo_over_path);
	soloBtn->move( 22, 20);
	QToolTip::add( soloBtn, trUtf8( "Solo" ) );
	connect(soloBtn, SIGNAL(clicked(Button*)), this, SLOT(click(Button*)));

	// delay fader
	delayFader = new PanFader(this);
	delayFader->move( 2, 35 );
	QToolTip::add( delayFader, trUtf8( "Delay" ) );
	connect( delayFader, SIGNAL( valueChanged(PanFader*) ), this, SLOT( delayFaderChanged(PanFader*) ) );

	// pan fader
	panFader = new PanFader(this);
	panFader->move( 2, 50 );
	QToolTip::add( panFader, trUtf8( "Pan" ) );
	connect( panFader, SIGNAL( valueChanged(PanFader*) ), this, SLOT( panFaderChanged(PanFader*) ) );


	PreferencesMng *pref = PreferencesMng::getInstance();
	QString family = pref->getMixerFontFamily().c_str();
	int size = pref->getMixerFontPointSize();
	QFont mixerFont( family, size );


	string mixerline_text_path = string(IMG_PATH).append("/img/mixerPanel/mixerline_text_background.png");
	QPixmap textBackground;
	ok = textBackground.load( mixerline_text_path .c_str() );
	if( ok == false ){
		qWarning( "MixerLine: Error loading pixmap" );
	}

	// peak volume label
	QFont font1( family, 10 );
	volumeLbl = new QLabel( this );
	volumeLbl->move( 2, 65 );
	volumeLbl->resize( 40, 20 );
	volumeLbl->setText( "0.0" );
	volumeLbl->setAlignment( AlignVCenter | AlignHCenter );
	volumeLbl->setBackgroundPixmap( textBackground );
	volumeLbl->setFont( font1 );
	QToolTip::add( volumeLbl, trUtf8( "Peak volume" ) );

	// instrument name widget
	nameWidget = new InstrumentNameWidget( this );
	nameWidget->move( 2, 85 );
	QToolTip::add( nameWidget, trUtf8( "Instrument name (double click to edit)" ) );
	connect( nameWidget, SIGNAL( clicked () ), this, SLOT( nameClicked() ) );

	// fader
	fader = new Fader(this);
	fader->move( 22, 85 );
	connect( fader, SIGNAL( valueChanged(Fader*) ), this, SLOT( faderChanged(Fader*) ) );

}




/**
 *
 */
MixerLine::~MixerLine() {
//	cout << "MixerLine destroy" << endl;
	delete fader;
}




/**
 *
 */
void MixerLine::updateMixerLine() {
	fader->updateFader();
	panFader->updateFader();
	delayFader->updateFader();
}





/**
 *
 */
void MixerLine::click(Button *ref) {
	Song *song = (Hydrogen::getInstance())->getSong();

	if (ref == muteBtn) {
		song->setModified(true);
		emit muteBtnClicked(this);
	}
	else if (ref == soloBtn) {
		song->setModified(true);
		emit soloBtnClicked(this);
	}
	else if (ref == loadBtn) {
		song->setModified(true);
		emit loadBtnClicked(this);
	}
	else if (ref == playSampleBtn) {
		emit noteOnClicked(this);
	}
}





/**
 *
 */
void MixerLine::rightClick(Button *ref)
{
	if (ref == playSampleBtn) {
		emit noteOffClicked(this);
	}

}



/**
 *
 */
void MixerLine::faderChanged(Fader *ref) {
	maxPeak = 0.0;
	char tmp[20];
	sprintf(tmp, "%#.1f", maxPeak);
	volumeLbl->setText(tmp);

	Song *song = (Hydrogen::getInstance())->getSong();
	song->setModified(true);
	emit volumeChanged(this);

	char faderPos[100];
	float value = ref->getValue() / 100.0;
	sprintf( faderPos, "%#.2f",  value);
	( HydrogenApp::getInstance() )->setStatusBarMessage( trUtf8( "Set instrument volume [").latin1() + string( faderPos ) + string( "]" ), 2000 );
}





/**
 *
 */
bool MixerLine::isMuteClicked() {
	return muteBtn->isPressed();
}





/**
 *
 */
void MixerLine::setMuteClicked(bool isClicked) {
	muteBtn->setPressed(isClicked);
}




/**
 *
 */
bool MixerLine::isSoloClicked() {
	return soloBtn->isPressed();
}




/**
 *
 */
void MixerLine::setSoloClicked(bool isClicked) {
	soloBtn->setPressed(isClicked);
}




/**
 *
 */
float MixerLine::getVolume() {
	return (fader->getValue() / 100.0);
}



/**
 *
 */
void MixerLine::setVolume(float value) {
	fader->setValue((int)(value * 100.0));
}



void MixerLine::setPeak_L( float peak ) {
	if (peak != getPeak_L() ) {
		fader->setPeak_L( peak );
		if (peak > maxPeak) {
			char tmp[20];
			sprintf(tmp, "%#.1f", peak);
			volumeLbl->setText(tmp);
			maxPeak = peak;
		}
	}
}



/**
 *
 */
float MixerLine::getPeak_L() {
	return fader->getPeak_L();
}




/**
 *
 */
void MixerLine::setPeak_R( float peak ) {
	if (peak != getPeak_R() ) {
		fader->setPeak_R( peak );
		if (peak > maxPeak) {
			char tmp[20];
			sprintf(tmp, "%#.1f", peak);
			volumeLbl->setText(tmp);
			maxPeak = peak;
		}
	}
}



/**
 *
 */
float MixerLine::getPeak_R() {
	return fader->getPeak_R();
}





/**
 *
 */
void MixerLine::nameClicked() {
	emit instrumentNameClicked(this);
}



/**
 *
 */
void MixerLine::panFaderChanged(PanFader *ref) {
	Song *song = ( Hydrogen::getInstance() )->getSong();
	song->setModified( true );
	emit panChanged( this );

	float panValue = ref->getValue();
	float pan_L = (100.0 - panValue) / 100.0;
	float pan_R = panValue / 100.0;

	char faderPos[100];
	sprintf( faderPos, "%#.2fL, %#.2fR",  pan_L, pan_R);
	( HydrogenApp::getInstance() )->setStatusBarMessage( trUtf8( "Set instrument pan [").latin1() + string( faderPos ) + string( "]"), 2000 );

	QToolTip::remove( panFader );
	QToolTip::add( panFader, QString("Pan ") + QString( faderPos ) );

}





/**
 *
 */
void MixerLine::delayFaderChanged(PanFader *ref) {
	Song *song = ( Hydrogen::getInstance() )->getSong();
	song->setModified( true );
	emit delayChanged( this );

	float delayValue = ref->getValue();

	char faderPos[100];
	sprintf( faderPos, "%#.2f",  delayValue / 100.0 );
	( HydrogenApp::getInstance() )->setStatusBarMessage( trUtf8( "Set instrument delay [").latin1() + string( faderPos ) + string( "]"), 2000 );

	QToolTip::remove( ref );
	QToolTip::add( ref, QString("Delay ") + QString( faderPos ) );

}



/**
 *
 */
int MixerLine::getPan() {
	return (int)( panFader->getValue() );
}



/**
 *
 */
void MixerLine::setPan(int value) {
	if ( value != panFader->getValue() ) {
		panFader->setValue( value );

		float pan_L = (100.0 - value) / 100.0;
		float pan_R = value / 100.0;
		char faderPos[100];
		sprintf( faderPos, "Pan %#.2fL, %#.2fR",  pan_L, pan_R);
		QToolTip::remove( panFader );
		QToolTip::add( panFader, QString( faderPos ) );
	}
}




float MixerLine::getDelayLevel() {
	return (delayFader->getValue() / 100.0);
}




void MixerLine::setDelayLevel(float value) {
	if ( (value * 100.0) != delayFader->getValue() ) {
		delayFader->setValue( (int)(value * 100.0) );
	}
}




void MixerLine::setPlayClicked( bool clicked ) {
	playSampleBtn->setPressed( clicked );
}







/**
 * Constructor
 */
MasterMixerLine::MasterMixerLine(QWidget* parent) : QWidget(parent) {
//	cout << "mixerLine init" << endl;

	setMinimumSize( width, height );
	setMaximumSize( width, height );
	resize( width, height );

	// Background image
	string background_path = string(IMG_PATH).append("/img/mixerPanel/mixerline_background.png");
	bool ok = background.load(background_path.c_str());
	if( ok == false ){
		qWarning( "Mixer: Error loading pixmap" );
	}
	setBackgroundPixmap(background);

	// MixerLine LABEL Background image
	QPixmap mixerLineLabelBackground;
	string mixerLineLabelBackground_path = string(IMG_PATH).append("/img/mixerPanel/mixerline_label_background.png");
	ok = mixerLineLabelBackground.load(mixerLineLabelBackground_path.c_str());
	if( ok == false ){
		qWarning( "MixerLine: Error loading pixmap" );
	}

	// Text background
	string mixerline_text_path = string(IMG_PATH).append("/img/mixerPanel/mixerline_text_background.png");
	QPixmap textBackground;
	ok = textBackground.load( mixerline_text_path .c_str() );
	if( ok == false ){
		qWarning( "MixerLine: Error loading pixmap" );
	}


	PreferencesMng *pref = PreferencesMng::getInstance();
	QString family = pref->getMixerFontFamily().c_str();
	QFont font1( family, 10 );
	// peak volume label
	leftVolumeLbl = new QLabel(this);
	leftVolumeLbl->move( 2, 65 );
	leftVolumeLbl->resize( 40, 20 );
	leftVolumeLbl->setText( "0.0" );
	leftVolumeLbl->setAlignment( AlignVCenter | AlignHCenter );
	leftVolumeLbl->setBackgroundPixmap( textBackground );
	leftVolumeLbl->setFont( font1 );
	QToolTip::add( leftVolumeLbl, trUtf8( "Peak volume (left channel)" ) );

	fader = new Fader( this );
	fader->move( 22, 85 );
	connect( fader, SIGNAL( valueChanged(Fader*) ), this, SLOT( faderChanged(Fader*) ) );

	int size = pref->getMixerFontPointSize();
	QFont mixerFont( family, size );

	InstrumentNameWidget *nameWidget;
	// instrument name widget
	nameWidget = new InstrumentNameWidget( this );
	nameWidget->move( 2, 85 );
	nameWidget->setText( trUtf8( "Master output" ) );
}




/**
 *
 */
MasterMixerLine::~MasterMixerLine() {
//	cout << "MixerLine destroy" << endl;
	maxPeak = 0.0;
}





/**
 *
 */
void MasterMixerLine::faderChanged(Fader *ref) {
	maxPeak = 0.0;

	char tmp[20];
	sprintf(tmp, "%#.1f", maxPeak);
	leftVolumeLbl->setText(tmp);
	fader->setValue(ref->getValue());

	emit volumeChanged(this);

	Song *song = (Hydrogen::getInstance())->getSong();
	song->setModified(true);

	char faderPos[100];
	float value = ref->getValue() / 100.0;
	sprintf( faderPos, "%#.2f",  value);
	( HydrogenApp::getInstance() )->setStatusBarMessage( trUtf8( "Set master volume [").latin1() + string( faderPos ) + string( "]" ), 2000 );

}




/**
 *
 */
float MasterMixerLine::getVolume() {
	return (fader->getValue() / 100.0);
}



/**
 *
 */
void MasterMixerLine::setVolume(float value) {
	fader->setValue((int)(value * 100.0));
//	rightFader->setValue((int)(value * 100.0));
}




/**
 *
 */
void MasterMixerLine::setPeak_L(float peak) {
	if ( peak != getPeak_L() ) {
		fader->setPeak_L(peak);
		if (peak > maxPeak) {
			char tmp[20];
			sprintf(tmp, "%#.1f", peak);
			leftVolumeLbl->setText(tmp);
			maxPeak = peak;
		}
	}
}




/**
 *
 */
float MasterMixerLine::getPeak_L() {
	return fader->getPeak_L();
}




/**
 *
 */
void MasterMixerLine::setPeak_R(float peak) {
	if ( peak != getPeak_R() ) {
		fader->setPeak_R(peak);

		if (peak > maxPeak) {
			char tmp[20];
			sprintf(tmp, "%#.1f", peak);
			leftVolumeLbl->setText(tmp);
			maxPeak = peak;
		}
	}
}





/**
 *
 */
float MasterMixerLine::getPeak_R() {
	return fader->getPeak_R();
}




/**
 *
 */
void MasterMixerLine::updateMixerLine() {
	fader->updateFader();
}




/////////////////////////////////////////



/**
 * Constructor
 */
FxMixerLine::FxMixerLine(QWidget* parent) : QWidget(parent) {
//	cout << "mixerLine init" << endl;

	setMinimumSize( width, height );
	setMaximumSize( width, height );
	resize( width, height );

	maxPeak = 0.0;

	// MixerLine Background image
	string mixerLineBackground_path = string(IMG_PATH).append("/img/mixerPanel/mixerline_background.png");
	bool ok = mixerLineBackground.load(mixerLineBackground_path.c_str());
	if( ok == false ){
		qWarning( "MixerLine: Error loading pixmap" );
	}
	setBackgroundPixmap(mixerLineBackground);

	// MixerLine LABEL Background image
	QPixmap mixerLineLabelBackground;
	string mixerLineLabelBackground_path = string(IMG_PATH).append("/img/mixerPanel/mixerline_label_background.png");
	ok = mixerLineLabelBackground.load(mixerLineLabelBackground_path.c_str());
	if( ok == false ){
		qWarning( "MixerLine: Error loading pixmap" );
	}

	string mixerline_text_path = string(IMG_PATH).append("/img/mixerPanel/mixerline_text_background.png");
	QPixmap textBackground;
	ok = textBackground.load( mixerline_text_path .c_str() );
	if( ok == false ){
		qWarning( "MixerLine: Error loading pixmap" );
	}


	// active button
	string activeBtn_on_path = string(IMG_PATH).append( "/img/mixerPanel/btn_on_on.png" );
	string activeBtn_off_path = string(IMG_PATH).append( "/img/mixerPanel/btn_on_off.png" );
	string activeBtn_over_path = string(IMG_PATH).append( "/img/mixerPanel/btn_on_over.png" );
	activeBtn = new ToggleButton( this, QSize( 18, 12 ), activeBtn_on_path, activeBtn_off_path, activeBtn_over_path);
	activeBtn->move( 2, 5 );
	QToolTip::add( activeBtn, trUtf8( "FX on/off") );
	connect( activeBtn, SIGNAL( clicked(Button*) ), this, SLOT( click(Button*) ) );

	PreferencesMng *pref = PreferencesMng::getInstance();

	// peak volume label
	QString family = pref->getMixerFontFamily().c_str();
	QFont font1( family, 10);
	volumeLbl = new QLabel( this );
	volumeLbl->move( 2, 65 );
	volumeLbl->resize( 40, 20 );
	volumeLbl->setText( "0.0" );
	volumeLbl->setAlignment( AlignVCenter | AlignHCenter );
	volumeLbl->setBackgroundPixmap( textBackground );
	volumeLbl->setFont( font1 );
//	QToolTip::add( volumeLbl, "Peak volume" );

	// fader
	fader = new Fader( this );
	fader->move( 22, 85 );
	connect( fader, SIGNAL( valueChanged(Fader*) ), this, SLOT( faderChanged(Fader*) ) );

	int size = pref->getMixerFontPointSize();
	QFont mixerFont( family, size );


	nameWidget = new InstrumentNameWidget( this );
	nameWidget->move( 2, 85 );
	nameWidget->setText( trUtf8( "Master output" ) );
}




/**
 *
 */
FxMixerLine::~FxMixerLine() {
//	cout << "MixerLine destroy" << endl;
	delete fader;
}





/**
 *
 */
void FxMixerLine::click(Button *ref) {
	Song *song = (Hydrogen::getInstance())->getSong();

	if (ref == activeBtn ) {
		song->setModified( true );
		emit activeBtnClicked( this );
	}
}




/**
 *
 */
void FxMixerLine::faderChanged(Fader *ref) {
	maxPeak = 0.0;
	char tmp[20];
	sprintf( tmp, "%#.1f", maxPeak );
	volumeLbl->setText( tmp );

	Song *song = ( Hydrogen::getInstance() )->getSong();
	song->setModified( true );
	emit volumeChanged( this );

}




/**
 *
 */
float FxMixerLine::getVolume() {
	return (fader->getValue() / 100.0);
}





/**
 *
 */
void FxMixerLine::setVolume( float value ) {
	fader->setValue( (int)(value * 100.0) );
}




/**
 *
 */
void FxMixerLine::setPeak_L( float peak ) {
	if (peak != getPeak_L() ) {
		fader->setPeak_L( peak );
		if (peak > maxPeak) {
			char tmp[20];
			sprintf(tmp, "%#.1f", peak);
			volumeLbl->setText(tmp);
			maxPeak = peak;
		}
	}
}



/**
 *
 */
float FxMixerLine::getPeak_L() {
	return fader->getPeak_L();
}





/**
 *
 */
void FxMixerLine::setPeak_R(float peak) {
	if (peak != getPeak_R() ) {
		fader->setPeak_R( peak );
		if (peak > maxPeak) {
			char tmp[20];
			sprintf(tmp, "%#.1f", peak);
			volumeLbl->setText(tmp);
			maxPeak = peak;
		}
	}
}





/**
 *
 */
float FxMixerLine::getPeak_R() {
	return fader->getPeak_R();
}




bool FxMixerLine::isFxActive() {
	return activeBtn->isPressed();
}




void FxMixerLine::setFxActive( bool active ) {
	activeBtn->setPressed( active );
}



/**
 *
 */
void FxMixerLine::updateMixerLine() {
	fader->updateFader();
}






////////////////////////////////



/**
 *
 */
DelayFxMixerLine::DelayFxMixerLine( QWidget* parent ) : FxMixerLine( parent ) {
//	cout << "DelayFXMixerLine INIT" << endl;
	// delay time fader
	delayTimeFader = new PanFader( this );
	delayTimeFader->move( 2, 35 );
	QToolTip::add( delayTimeFader, trUtf8( "Delay time" ) );
	connect( delayTimeFader, SIGNAL( valueChanged(PanFader*) ), this, SLOT( delayTimeFaderChanged(PanFader*) ) );

	// feedback fader
	feedbackFader = new PanFader( this );
	feedbackFader->move( 2, 50 );
	QToolTip::add( feedbackFader, trUtf8( "Feedback" ) );
	connect( feedbackFader, SIGNAL( valueChanged(PanFader*) ), this, SLOT( feedbackFaderChanged(PanFader*) ) );

}



/**
 *
 */
void DelayFxMixerLine::feedbackFaderChanged( PanFader * ref ) {
	char faderPos[100];
	float value = ref->getValue() / 100.0;
	sprintf( faderPos, "%#.2f",  value);
	( HydrogenApp::getInstance() )->setStatusBarMessage( trUtf8( "Set delay feedback [").latin1() + string( faderPos ) + string( "]" ), 2000 );

	QToolTip::remove( ref );
	QToolTip::add( ref, QString("Feedback ") + QString( faderPos ) );

	emit feedbackChanged( this );
}





/**
 *
 */
void DelayFxMixerLine::delayTimeFaderChanged( PanFader * ref ) {
	char faderPos[100];
	int value = (int)( ( ref->getValue() * 7.0 / 100.0  ) + 1 ) * 2;
	sprintf( faderPos, "%d",  value);
	( HydrogenApp::getInstance() )->setStatusBarMessage( trUtf8( "Set delay time [").latin1() + string( faderPos ) + string( "]" ), 2000 );

	QToolTip::remove( ref );
	QToolTip::add( ref, trUtf8( "Delay time ") + QString( faderPos ) );

	emit delayTimeChanged( this );
}




/**
 *
 */
float DelayFxMixerLine::getFeedbackLevel() {
	return feedbackFader->getValue() / 100.0;
}




/**
 *
 */
void DelayFxMixerLine::setFeedbackLevel( float level ) {
	feedbackFader->setValue( level * 100.0 );
}




/**
 *
 */
float  DelayFxMixerLine::getDelayTime() {
	float value = ( ( delayTimeFader->getValue() / 100.0 * 7 ) + 1 ) * 2;
	return value;
}

//16: 100 = x : valeu


/**
 *
 */
void DelayFxMixerLine::setDelayTime( float time ) {
	float value = ( 100.0 * time / 15.0 ) + 1;
	delayTimeFader->setValue( value );
}



/**
 *
 */
void DelayFxMixerLine::updateMixerLine() {
	FxMixerLine::updateMixerLine();
	feedbackFader->updateFader();
	delayTimeFader->updateFader();
}




/**
 *
 */
InstrumentNameWidget::InstrumentNameWidget(QWidget* parent)  : QWidget(parent)
{
//	cout << "InstrumentNameWidget() INIT" << endl;

	m_bChanged = true;
	m_nWidgetWidth = 18;
	m_nWidgetHeight = 100;

	PreferencesMng *pref = PreferencesMng::getInstance();
	QString family = pref->getMixerFontFamily().c_str();
	int size = pref->getMixerFontPointSize();
	m_mixerFont.setFamily( family );
	m_mixerFont.setPointSize( size );
//	m_mixerFont.setBold( true );
//	m_mixerFont.setItalic( true );

	// MixerLine LABEL Background image
	QPixmap mixerLineLabelBackground;
	string mixerLineLabelBackground_path = string(IMG_PATH).append( "/img/mixerPanel/mixerline_label_background.png" );
 bool ok = m_background.load( mixerLineLabelBackground_path.c_str() );
	if( ok == false ){
		qWarning( "InstrumentNameWidget: Error loading pixmap" );
	}

	m_temp.resize( m_nWidgetWidth, m_nWidgetHeight );
//	m_background.resize( m_nWidgetWidth, m_nWidgetHeight );
	this->resize( m_nWidgetWidth, m_nWidgetHeight );
}


/**
 *
 */
InstrumentNameWidget::~InstrumentNameWidget()
{
//	cout << "InstrumentNameWidget() DESTROY" << endl;
}



/**
 * Paint method
 */
void InstrumentNameWidget::paintEvent( QPaintEvent* ev ) {
	if (!isVisible()) {
		return;
	}

	if (m_bChanged) {
		m_bChanged = false;
//		cout << "----------- InstrumentNameWidget PAINT --------------" << endl;

		bitBlt(&m_temp, 0, 0, &m_background, 0, 0, m_nWidgetWidth, m_nWidgetHeight, CopyROP);
		QPainter p( &m_temp );
		p.setPen( QColor(0, 0, 0) );
		p.setFont( m_mixerFont );
		p.rotate( -90 );
		p.drawText( -95, 15, m_sInstrName );

		setErasePixmap( m_temp );

		// copio l'immagine definitiva
		bitBlt( this, 0, 0, &m_temp, 0, 0, m_nWidgetWidth, m_nWidgetHeight, CopyROP );
	}
}




/**
 *
 */
void InstrumentNameWidget::setText( QString text )
{
	if (m_sInstrName != text ) {
		m_sInstrName = text;
		m_bChanged = true;
		update();
	}
}

/**
 *
 */
QString InstrumentNameWidget::text()
{
	return m_sInstrName;
}



