/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MixerLine.h,v 1.33 2003/09/22 15:23:53 comix Exp $
 *
 */


#ifndef MIXERLINE_H
#define MIXERLINE_H

#include "qwidget.h"
#include "qlabel.h"
#include "qtooltip.h"

#include <hydrogen/Hydrogen.h>
#include <hydrogen/PreferencesMng.h>

#include "Fader.h"
#include "Button.h"
#include "InstrumentPropertiesDialog.h"

/**
 *
 */
class InstrumentNameWidget : public QWidget
{
	Q_OBJECT
	public:
		InstrumentNameWidget(QWidget* parent);
		~InstrumentNameWidget();

		void paintEvent(QPaintEvent *ev);

		void setText(QString text);
		QString text();

		void mouseDoubleClickEvent ( QMouseEvent * e ) {
			emit clicked();
		}

	signals:
		void clicked();

	private:
		QPixmap m_background;
		QPixmap m_temp;
		bool m_bChanged;
		int m_nWidgetWidth;
		int m_nWidgetHeight;
		QString m_sInstrName;
		QFont m_mixerFont;
};




/**
 * A mixer strip
 */
class MixerLine: public QWidget
{
	Q_OBJECT
	public:
		/** Constructor */
		MixerLine(QWidget* parent);

		/** Destructor */
		~MixerLine();

		void updateMixerLine();

		bool isMuteClicked();
		void setMuteClicked(bool isClicked);

		bool isSoloClicked();
		void setSoloClicked(bool isClicked);

		float getVolume();
		void setVolume(float value);

		void setPeak_L( float peak );
		float getPeak_L();

		void setPeak_R( float peak );
		float getPeak_R();

		void setName(string name) {     nameWidget->setText( QString( name.c_str() ) );        }
		string getName() {      return string( nameWidget->text().latin1() );        }

		int getPan();
		void setPan(int value);

		int getActivity() {	return activity;	}
		void setActivity( uint value ) {	activity = value;	}

		void setPlayClicked( bool clicked );

		float getDelayLevel();
		void setDelayLevel(float value);

	signals:
		void muteBtnClicked(MixerLine *ref);
		void soloBtnClicked(MixerLine *ref);
		void loadBtnClicked(MixerLine *ref);
		void volumeChanged(MixerLine *ref);
		void instrumentNameClicked(MixerLine *ref);
		void noteOnClicked(MixerLine *ref);
		void noteOffClicked(MixerLine *ref);
		void panChanged(MixerLine *ref);
		void delayChanged(MixerLine *ref);

	public slots:
		void click(Button *ref);
		void rightClick(Button *ref);
		void faderChanged(Fader * ref);
		void panFaderChanged(PanFader * ref);
		void delayFaderChanged(PanFader * ref);
		void nameClicked();

	private:
		static const uint width = 42;
		static const uint height = 190;

		uint activity;
		float maxPeak;
		Fader *fader;
		PanFader *panFader;
		PanFader *delayFader;
		QLabel *volumeLbl;
		InstrumentNameWidget *nameWidget;
		ToggleButton *muteBtn;
		ToggleButton *soloBtn;
		Button *loadBtn;
		Button *playSampleBtn;
		QPixmap mixerLineBackground;

};




/**
 *
 */
class MasterMixerLine: public QWidget {
	Q_OBJECT
	public:
		/** Constructor */
		MasterMixerLine(QWidget* parent);

		/** Destructor */
		~MasterMixerLine();

		void updateMixerLine();

		float getVolume();
		void setVolume(float value);

		void setPeak_L(float peak);
		float getPeak_L();

		void setPeak_R(float peak);
		float getPeak_R();


	signals:
		void volumeChanged(MasterMixerLine *ref);

	public slots:
		void faderChanged(Fader * ref);

	private:
		static const uint width = 42;
		static const uint height = 190;

		float maxPeak;
		QPixmap background;
		Fader *fader;
		QLabel *leftVolumeLbl;

};




/**
 * Mixer strip for FX
 */
class FxMixerLine: public QWidget {
	Q_OBJECT
	public:
		/** Constructor */
		FxMixerLine(QWidget* parent);

		/** Destructor */
		~FxMixerLine();

		void updateMixerLine();

		float getVolume();
		void setVolume(float value);

		void setPeak_L(float peak);
		float getPeak_L();

		void setPeak_R(float peak);
		float getPeak_R();

		void setName(string name) {     nameWidget->setText(QString(name.c_str()));        }
		string getName() {      return string(nameWidget->text().latin1());        }

		bool isFxActive();
		void setFxActive( bool active );

	signals:
		void volumeChanged( FxMixerLine *ref );
		void instrumentNameClicked( FxMixerLine *ref );
		void activeBtnClicked( FxMixerLine *ref );

	public slots:
		void click(Button *ref);
		void faderChanged(Fader * ref);

	private:
		static const uint width = 42;
		static const uint height = 190;

		float maxPeak;
		Fader *fader;
		QLabel *volumeLbl;
		InstrumentNameWidget *nameWidget;
		QPixmap mixerLineBackground;
		ToggleButton *activeBtn;

};





/**
 * Mixerline for Delay fx
 */
class DelayFxMixerLine : public FxMixerLine {
	Q_OBJECT
	public:
		/** Constructor */
		DelayFxMixerLine(QWidget* parent);

		void updateMixerLine();

		float getFeedbackLevel();
		void setFeedbackLevel(float level);

		float getDelayTime();
		void setDelayTime( float time );

	signals:
		void feedbackChanged(DelayFxMixerLine *ref);
		void delayTimeChanged(DelayFxMixerLine *ref);

	public slots:
		void feedbackFaderChanged( PanFader * ref );
		void delayTimeFaderChanged( PanFader * ref );

	private:
		PanFader *feedbackFader;
		PanFader *delayTimeFader;

};

#endif

