/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: PatternEditor.cpp,v 1.56 2003/09/26 16:02:45 comix Exp $
 *
 */


#include "PatternEditor.h"


/**
 * Constructor
 */
PatternEditor::PatternEditor(QWidget* parent)  : QWidget(parent) {
//	cout << "PatternEditor INIT" << endl;

	m_nGridWidth = 3;
	m_nGridHeight = 12;

	m_nEditorWidth = 5 + m_nGridWidth * MAX_NOTES + 5;
	m_nEditorHeight = m_nGridHeight * MAX_INSTRUMENTS + 52;

	currentPattern = NULL;
	selectedInstrument = 0;
	m_nResolution = 8;
	m_bUseTriplets = false;
	changed = true;
	notesChanged = true;

	// Prepare pixmaps
	string background_path = IMG_PATH;
	background_path.append( "/img/patternEditor/editor_background.png" );
	bool ok = background.load( background_path.c_str() );
	if( ok == false ){
		qWarning( "PatternEditor: Error loading pixmap" );
	}
	createBackground( &background );
	setBackgroundPixmap( background );

	resize( m_nEditorWidth, m_nEditorHeight );
	setMinimumSize( m_nEditorWidth, m_nEditorHeight );
	setMaximumSize( m_nEditorWidth, m_nEditorHeight );

	background.resize(m_nEditorWidth, m_nEditorHeight );
	temp.resize(m_nEditorWidth, m_nEditorHeight );
	gridAndNotes.resize( m_nEditorWidth, m_nEditorHeight );

	(Hydrogen::getInstance())->addEngineListener( this );
}





/**
 * Destructor
 */
PatternEditor::~PatternEditor()
{
//	cout << "PatternEditor DESTROY" << endl;
}




/**
 *
 */
void PatternEditor::updateEditor()
{
	if(!isVisible()) {
		return;
	}

	Hydrogen* engine = Hydrogen::getInstance();

	// check engine state
	int state = engine->getState();
	if ( (state != READY) && (state != PLAYING) ) {
		cerr << "TODO: skip pattern editor update (state shoud be READY or PLAYING" << endl;
		return;
	}

	Pattern* enginePattern = engine->getCurrentPattern();
	if (enginePattern != NULL) {

		if (currentPattern != enginePattern) {
			currentPattern = enginePattern;
			createBackground(&background);
			changed = true;
			notesChanged = true;
		}
	}
	else {
		if (currentPattern != enginePattern) {
			currentPattern = enginePattern;
			createBackground(&background);
			changed = true;
			notesChanged = true;
		}
	}

	// redraw all
	update();
}





/**
 *
 */
void PatternEditor::mousePressEvent(QMouseEvent *ev) {
	if (currentPattern == NULL) {
		return;
	}

	int row = MAX_INSTRUMENTS - 1 - (ev->y()  / (int)m_nGridHeight);
	if (row >= MAX_INSTRUMENTS) {
		return;
	}

//	cout << "mouse press event (x=" << ev->x() << ")" << endl;
//	cout << "m_nResolution = " << m_nResolution << endl;


	int width = m_nGridWidth * ( MAX_NOTES / m_nResolution);
//	cout << "width = " << width << endl;

	int x = ev->x();
	int column;
	column = x - 10 + (width / 2);
//	cout << "column1 = " << column << endl;

	column = column / width;
//	cout << "column2 = " << column << endl;

	column = column * (MAX_NOTES / m_nResolution);
//	cout << "column3 = " << column << endl;



	if (column >= MAX_NOTES) {
		update();
		return;
	}


	if (row < 0) {	// click on Note Properties Ruler
		int ruler_y_start = m_nGridHeight * MAX_INSTRUMENTS;
		int ruler_y_end = ruler_y_start + 52;

		float val = 0;
		if (ev->y() < ruler_y_end) {
			val = ruler_y_end - ev->y();
		}
		if (val > 50.0) {
			val = 50.0;
		}
		else if (val < 0.0) {
			val = 0.0;
		}
		val = val * 2;
		val = val / 100.0;
		SequenceList *sequenceList = currentPattern->getSequenceList();

		Sequence *seq = sequenceList->get(selectedInstrument);
		for (uint j = 0; j < MAX_NOTES; j++) {
			Note *note = seq->noteList[j];
			if ( (note != NULL) && ((int)(note->getPosition()) == column) ){
				note->setVelocity(val);
				changed = true;
				notesChanged = true;
				update();
				Song *song = (Hydrogen::getInstance())->getSong();
				song->setModified(true);

				char valueChar[100];
				sprintf( valueChar, "%#.2f",  val);
				( HydrogenApp::getInstance() )->setStatusBarMessage( string("Set note velocity [") + string( valueChar ) + string( "]" ), 2000 );

				break;
			}
		}
		return;
	}
	else {	// click on Pattern editor

		if (selectedInstrument != (uint)row) {
			selectedInstrument = row;
			createBackground(&background);
			changed = true;
			notesChanged = true;
		}
/*
		if (ev->x() <= 90) {	// click on instrument name

			update();
			if (ev->button() == RightButton ) {
				functionPopup->popup( QPoint( ev->globalX(), ev->globalY() ) );
			}
			return;
		}
*/
		( Hydrogen::getInstance() )->lockEngine();	// lock the audio engine
		SequenceList *sequenceList = currentPattern->getSequenceList();
		Sequence *seq = sequenceList->get(row);

		if (seq->noteList[column] != NULL) {
			Note *note = seq->noteList[column];
			delete note;
			seq->noteList[column] = NULL;
			notesChanged = true;
		}
		else {
			// create the new note
			uint position = column;
			float velocity = 0.8;
			float pan_L = 1.0;
			float pan_R = 1.0;
			Note *note = new Note(position, velocity, pan_L, pan_R);
			Song *song = (Hydrogen::getInstance())->getSong();
			Instrument *instrRef = (song->getInstrumentList())->get(row);
			note->setInstrument(instrRef);
			seq->noteList[column] = note;
			notesChanged = true;

			// hear note
			PreferencesMng *pref = PreferencesMng::getInstance();
			if ( pref->getHearNewNotes() ) {
				Note *note2 = new Note(position, velocity, pan_L, pan_R);
				note2->setInstrument(instrRef);
				( Hydrogen::getInstance() )->noteOn( note2 );
			}
		}
		changed = true;
		Song *song = (Hydrogen::getInstance())->getSong();
		song->setModified(true);
		( Hydrogen::getInstance() )->unlockEngine(); // unlock the audio engine

		update();
		m_pPatternEditorInstrumentList->updateEditor();
	}
}





/**
 * Draw a note
 */
void PatternEditor::drawNote(Note *note, uint nSequence, QPixmap *pixmap) {
	QPainter p(pixmap);
	QColor black(0, 0, 0);

	p.setPen(black);

	uint pos = note->getPosition() % MAX_NOTES;
	uint x_pos = 10 + pos * m_nGridWidth - 3;

	uint y_pos = (MAX_INSTRUMENTS  * m_nGridHeight) - ((nSequence + 1) * m_nGridHeight) + 3;

	// draw the "dot"
	p.drawLine(x_pos + 3, y_pos, x_pos + 6, y_pos + 3);
	p.drawLine(x_pos + 3, y_pos, x_pos, y_pos + 3);
	p.drawLine(x_pos + 3, y_pos + 6, x_pos + 6, y_pos + 3);
	p.drawLine(x_pos, y_pos + 3, x_pos + 3, y_pos + 6);

	p.drawLine(x_pos + 3, y_pos + 1, x_pos + 5, y_pos + 3);
	p.drawLine(x_pos + 3, y_pos + 1, x_pos + 1, y_pos + 3);
	p.drawLine(x_pos + 3, y_pos + 5, x_pos + 5, y_pos + 3);
	p.drawLine(x_pos + 1, y_pos + 3, x_pos + 3, y_pos + 5);

	p.drawLine(x_pos + 3, y_pos + 2, x_pos + 4, y_pos + 3);
	p.drawLine(x_pos + 3, y_pos + 2, x_pos + 2, y_pos + 3);
	p.drawLine(x_pos + 3, y_pos + 4, x_pos + 4, y_pos + 3);
	p.drawLine(x_pos + 2, y_pos + 3, x_pos + 3, y_pos + 4);

	if ( nSequence == selectedInstrument ) {	// draw the velocity
		QColor center(80, 97, 117);
		QColor side(123, 149, 180);

		uint line_start = m_nGridHeight * MAX_INSTRUMENTS + 1;
		uint line_end = line_start + 50;

		uint velocity = (uint)(note->getVelocity() * 50.0);
		line_start = line_end - velocity;

		p.setPen(side);
		p.drawLine(x_pos + 2, line_start, x_pos + 2, line_end);	// left side
		p.drawLine(x_pos + 4, line_start, x_pos + 4, line_end);	// right side

		p.setPen(center);
		p.drawLine(x_pos + 3, line_start, x_pos + 3, line_end);	// center

	}
}





/**
 *
 */
void PatternEditor::createBackground(QPixmap *pixmap) {
//	cout << "*** PatternEditor: createBackground" << endl;
	pixmap->fill();		// fill the pixmap with white color

	QFont font;
	font.setPointSize(10);

	QPainter p(pixmap);
	p.setFont(font);

	// start y of velocity ruler
	uint velRuler_y_start = m_nGridHeight * MAX_INSTRUMENTS;

	// horizontal lines (only fill color)
	for (uint i = 0; i < MAX_INSTRUMENTS; i++) {
		uint y = m_nGridHeight * i;

		if ( ((MAX_INSTRUMENTS -1) - i) == selectedInstrument) {
			// highlighted instrument line
//			p.setPen(QColor(219, 243, 255));	// color for highlighted instrument line
			p.setPen(QColor(209, 233, 245));	// color for highlighted instrument line
			// fill line
			for (uint j = 1; j < m_nGridHeight; j++) {
				p.drawLine( 0, y + j, (10 + MAX_NOTES * m_nGridWidth), y + j);
			}
		}
		else {
			// fill line
			for (uint j = 1; j < m_nGridHeight; j++) {
				if (i % 2 != 0) {
					p.setPen(QColor(240, 240, 255));
					p.drawLine( 0, y + j, (10 + MAX_NOTES * m_nGridWidth), y + j);
				}
			}
		}
	}

	// vertical lines
	QColor res_1(0, 0, 0);
	QColor res_2(100, 100, 100);
	QColor res_3(150, 150, 150);
	QColor res_4(200, 200, 200);
	QColor res_5(230, 230, 230);

	QColor test(200, 0, 0);

	p.setPen(res_1);
	p.drawLine(0, 0, 0, velRuler_y_start + 52);

	int nBase;
	if (m_bUseTriplets) {
		nBase = 3;
	}
	else {
		nBase = 4;
	}

	int n4th = 4 * MAX_NOTES / (nBase * 4);
	int n8th = 4 * MAX_NOTES / (nBase * 8);
	int n16th = 4 * MAX_NOTES / (nBase * 16);
	int n32th = 4 * MAX_NOTES / (nBase * 32);
	int n64th = 4 * MAX_NOTES / (nBase * 64);


	for (uint i = 0; i < MAX_NOTES + 1; i++) {
		uint x = 10 + i * m_nGridWidth;

		if ( (i % n4th) == 0 ) {
			if (m_nResolution >= 4) {
				p.setPen(res_1);
				p.drawLine(x, 0, x, velRuler_y_start + 52);
			}
		}
		else if ( (i % n8th) == 0 ) {
			if (m_nResolution >= 8) {
				p.setPen(res_2);
				p.drawLine(x, 0, x, velRuler_y_start + 52);
			}
		}
		else if ( (i % n16th) == 0 ) {
			if (m_nResolution >= 16) {
				p.setPen(res_3);
				p.drawLine(x, 0, x, velRuler_y_start + 52);
			}
		}
		else if ( (i % n32th) == 0 ) {
			if (m_nResolution >= 32) {
				p.setPen(res_4);
				p.drawLine(x, 0, x, velRuler_y_start + 52);
			}
		}
		else if ( (i % n64th) == 0 ) {
			if (m_nResolution >= 64) {
				p.setPen(res_5);
				p.drawLine(x, 0, x, velRuler_y_start + 52);
			}
		}
	}



/*
	uint n8thTriplet = MAX_NOTES / (3 * 8) * 4;
	for (uint i = 0; i < MAX_NOTES + 1; i++) {
		uint x = 10 + i * m_nGridWidth;

		if ( (i % n8thTriplet) == 0 ) {
			p.setPen(test);
			p.drawLine(x, 0, x, velRuler_y_start + 52);
		}
	}
*/
/*
	for (uint i = 0; i < MAX_NOTES + 1; i++) {
		uint x = 10 + i * m_nGridWidth;
		if ((i % MAX_NOTES) == 0) {
			if (m_nResolution > MAX_NOTES) {
				continue;
			}
			p.setPen(res_1);
		}
		else if ((i % (nQuarter * 8) ) == 0) {
			if (m_nResolution > 32) {
				continue;
			}
			p.setPen(res_2);
		}
		else if ( ( i % (nQuarter * 4) ) == 0) {
			if (m_nResolution > 16) {
				continue;
			}
			p.setPen(res_3);
		}
		else if ((i % nQuarter * 2) == 0) {
			if (m_nResolution > 8) {
				continue;
			}
			p.setPen(res_4);
		}
		else if ((i % (nQuarter)) == 0) {
			if (m_nResolution > 4) {
				continue;
			}
			p.setPen(res_4);
		}
		else if ( (i % (nQuarter / 2) ) == 0) {
			if (m_nResolution > 2) {
				continue;
			}
			p.setPen(res_5);
		}
		else if ((i % (nQuarter / 4)) == 0) {
			if (m_nResolution > 1) {
				continue;
			}
			p.setPen(res_5);
		}
		else {
			continue;
//			cout << "m_nResolution???uh?" << endl;
//			p.setPen(QColor(230, 230, 230));
		}
		p.drawLine(x, 0, x, velRuler_y_start + 52);
	}
*/

	// horizontal lines
	p.setPen(QColor(0, 0, 0));
	for (uint i = 0; i < MAX_INSTRUMENTS; i++) {
		uint y = m_nGridHeight * i;
		p.drawLine( 0, y, (10 + MAX_NOTES * m_nGridWidth), y);
	}
	// end of instruments

	p.setPen(QColor(0, 0, 0));
	p.drawLine( 0, m_nGridHeight * MAX_INSTRUMENTS, (10 + MAX_NOTES * m_nGridWidth), m_nGridHeight * MAX_INSTRUMENTS);


	// Velocity ruler

	// end of the velocity ruler
	p.setPen(QColor(0, 0, 0));
	p.drawLine( 0, velRuler_y_start + 52, (10 + MAX_NOTES * m_nGridWidth), velRuler_y_start + 52);

}





/**
 * Paint method
 */
void PatternEditor::paintEvent( QPaintEvent*) {
	if (!isVisible()) {
		return;
	}

	if (changed) {
		changed = false;
//		cout << "----------- PatternEditor PAINT --------------" << endl;
		// ridisegno tutto solo se sono cambiate le note
		if (notesChanged) {
			bitBlt(&gridAndNotes, 0, 0, &background, 0, 0, m_nEditorWidth, m_nEditorHeight, CopyROP);
			drawPattern(&gridAndNotes);
			notesChanged = false;
		}
		bitBlt(&temp, 0, 0, &gridAndNotes, 0, 0, m_nEditorWidth, m_nEditorHeight, CopyROP);


		setErasePixmap(temp);

		// copio l'immagine definitiva
		bitBlt(this, 0, 0, &temp, 0, 0, m_nEditorWidth, m_nEditorHeight, CopyROP);
	}
}





/**
 *
 */
void PatternEditor::drawPattern(QPixmap *pixmap)
{
	Hydrogen *engine = Hydrogen::getInstance();
	Pattern *currentPattern = engine->getCurrentPattern();

	if (currentPattern == NULL) {
		return;
	}
//	cout << "*** patternEditor:drawpattern" << endl;

	SequenceList *sequenceList = currentPattern->getSequenceList();

	for (uint i = 0; i < sequenceList->getSize(); i++)
	{
		Sequence *seq = sequenceList->get(i);
		for (uint j = 0; j < MAX_NOTES; j++)
		{
			Note *note = seq->noteList[j];
			if (note != NULL)
			{
				drawNote(note, i, pixmap);
			}
		}
	}
}




/**
 * show event
 */
void PatternEditor::showEvent ( QShowEvent *ev ) {
	updateEditor();
//	updateStart(true);
}




/**
 * hide event
 */
void PatternEditor::hideEvent ( QHideEvent *ev ) {
//	updateStart(false);
}




/**
 *
 */
void PatternEditor::setResolution(uint res, bool bUseTriplets) {
	this->m_nResolution = res;
	this->m_bUseTriplets = bUseTriplets;

	// redraw all
	createBackground(&background);
	changed = true;
	notesChanged = true;
	update();
}



/**
 * This method is called from another thread (audio engine)
 */
void PatternEditor::patternChanged() {
	H2TextEvent *ev = new H2TextEvent("patternChanged");
	QApplication::postEvent(this, ev);

}



/**
 *
 */
void PatternEditor::customEvent( QCustomEvent *ev ) {
	if ( ev->type() != H2_TEXT_EVENT ) {	// Must be a H2TextEvent
		return;
	}
	string message = ((H2TextEvent*)ev)->getText();

	if (message == "patternChanged") {
		updateEditor();
	}

}



///
///
///
void PatternEditor::setSelectedInstrument(int nInstr)
{
	selectedInstrument = nInstr;
	createBackground(&background);
	changed = true;
	notesChanged = true;
}






///////////////////////////////////



/**
 *
 */
PatternEditorRuler::PatternEditorRuler(QWidget* parent)  : QWidget(parent) {
//	cout << "PatternEditorRuler INIT" << endl;

	m_nGridWidth = 3;

	m_nRulerWidth = 5 + m_nGridWidth * MAX_NOTES + 5;
	m_nRulerHeight = 10;

	setMinimumSize( m_nRulerWidth, m_nRulerHeight );
	setMaximumSize( m_nRulerWidth, m_nRulerHeight );
	resize( m_nRulerWidth, m_nRulerHeight );

	temp.resize( m_nRulerWidth, m_nRulerHeight );

	string tickPosition_path = string( IMG_PATH ) + string( "/img/patternEditor/tickPosition.png" );
	bool ok = tickPosition.load(tickPosition_path.c_str());
	if( ok == false ){
		qWarning( "PatternEditor: Error loading pixmap" );
	}

	background.resize( m_nRulerWidth, m_nRulerHeight );
	background.fill();		// fill the pixmap with white color
	QFont font;
	font.setPointSize(10);
	QPainter p( &background );
	p.setFont(font);
	p.setPen( QColor( 0, 0, 0 ) );
	p.drawLine( 0, 0, m_nRulerWidth, 0 );
	p.drawLine( 0, 0, 0, m_nRulerHeight );
	p.drawLine( m_nRulerWidth - 1, 0, m_nRulerWidth - 1, m_nRulerHeight );

	uint nQuarter = 48;

	uint x = 10 + 0 * m_nGridWidth;
	p.drawText( x - 2, 9, "1" );

	x = 10 + nQuarter * m_nGridWidth;
	p.drawText( x - 2, 9, "2" );

	x = 10 + nQuarter * 2 * m_nGridWidth;
	p.drawText( x - 2, 9, "3" );

	x = 10 + nQuarter * 3 * m_nGridWidth;
	p.drawText( x - 2, 9, "4" );

	setBackgroundPixmap(background);

	changed = true;

	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(updateEditor()));
}





/**
 *
 */
PatternEditorRuler::~PatternEditorRuler() {
//	cout << "PatternEditorRuler DESTROY" << endl;
}





/**
 *
 */
void PatternEditorRuler::updateStart(bool start) {
	if (start) {
//		cout << "pattern editor ruler timer start" << endl;
		timer->start(100);	// update ruler at 10 fps
	}
	else {
//		cout << "pattern editor ruler timer stop" << endl;
		timer->stop();
	}
}




/**
 * show event
 */
void PatternEditorRuler::showEvent ( QShowEvent *ev ) {
	updateEditor();
	updateStart(true);
}




/**
 * hide event
 */
void PatternEditorRuler::hideEvent ( QHideEvent *ev ) {
	updateStart(false);
}






/**
 *
 */
void PatternEditorRuler::updateEditor() {
	static uint oldNTicks = 0;

	Hydrogen* engine = Hydrogen::getInstance();

	int state = engine->getState();
	if ( state == PLAYING ) {
		nTicks = engine->getTickPosition();
	}
	else {
		nTicks = 0;
	}

	if (oldNTicks != nTicks) {
		// redraw all
		changed = true;
		update();
	}
	oldNTicks = nTicks;
}



/**
 * Paint method
 */
void PatternEditorRuler::paintEvent( QPaintEvent*) {
	if (!isVisible()) {
		return;
	}

	if (changed) {
//		cout << "PatternEditorRuler PAINT" << endl;

		bitBlt(&temp, 0, 0, &background, 0, 0, m_nRulerWidth, m_nRulerHeight, CopyROP);

		// tickPosition
		uint x = 10 + nTicks * m_nGridWidth - 5;
		bitBlt( &temp, x, 0, &tickPosition, 0, 0, 11, 8, CopyROP );
		setErasePixmap(temp);

		// copio l'immagine definitiva
		bitBlt(this, 0, 0, &temp, 0, 0, 18, 100, CopyROP);
		changed = false;
	}
}
















//////////////////////////////////



///
/// Constructor
///
PatternEditorInstrumentList::PatternEditorInstrumentList( QWidget *parent, PatternEditor *pPatternEditor ) : QWidget( parent )
{
//	cout << "PatternEditorInstrumentList INIT" << endl;
	m_pPatternEditor = pPatternEditor;
	m_nEditorWidth = 100;
	m_nEditorHeight = m_nGridHeight * MAX_INSTRUMENTS;
	changed = true;

	background.resize(m_nEditorWidth, m_nEditorHeight );
	temp.resize(m_nEditorWidth, m_nEditorHeight );

	createBackground( &background );

	resize( m_nEditorWidth, m_nEditorHeight );
	setMinimumSize( m_nEditorWidth, m_nEditorHeight );
	setMaximumSize( m_nEditorWidth, m_nEditorHeight );


	string genericIcon_path = IMG_PATH;
	genericIcon_path.append( "/img/songEditor/patternIcon.png" );
	genericIcon.load( genericIcon_path.c_str() );

	// Popup menu
	functionPopup  = new QPopupMenu( this, "patternSequence_patternPopupMenu" );
	functionPopup->insertItem( genericIcon, "Clear notes",  this, SLOT( clearNotes() ) );
	functionPopup->insertItem( genericIcon, "Fill notes",  this, SLOT( fillNotes() ) );

//	timer = new QTimer(this);
//	connect(timer, SIGNAL(timeout()), this, SLOT(updateEditor()));


	(Hydrogen::getInstance())->addEngineListener( this );
}




///
/// Destructor
///
PatternEditorInstrumentList::~PatternEditorInstrumentList()
{
//	cout << "PatternEditorInstrumentList DESTROY" << endl;
}




/**
 * Paint method
 */
void PatternEditorInstrumentList::paintEvent( QPaintEvent*) {
	if (!isVisible()) {
		return;
	}

	if (changed) {
		changed = false;
		cout << "----------- PatternEditorInstrumentList PAINT --------------" << endl;
		bitBlt(&temp, 0, 0, &background, 0, 0, m_nEditorWidth, m_nEditorHeight, CopyROP);

		setErasePixmap(temp);

		// copio l'immagine definitiva
		bitBlt(this, 0, 0, &temp, 0, 0, m_nEditorWidth, m_nEditorHeight, CopyROP);
	}
}





/**
 *
 */
void PatternEditorInstrumentList::createBackground(QPixmap *pixmap) {
	cout << "[PatternEditorInstrumentList::createBackground()]" << endl;
	pixmap->fill();		// fill the pixmap with white color

	QFont font;
	font.setPointSize(10);

	QPainter p(pixmap);
	p.setFont(font);

	Song *song = (Hydrogen::getInstance())->getSong();
	InstrumentList *instrList = song->getInstrumentList();

	// horizontal lines (only fill color)
	for (uint i = 0; i < MAX_INSTRUMENTS; i++) {
		uint y = m_nGridHeight * i;

		if ( ((MAX_INSTRUMENTS -1) - i) == (uint)m_pPatternEditor->getSelectedInstrument() ) {
			// highlighted instrument line
//			p.setPen(QColor(219, 243, 255));	// color for highlighted instrument line
			p.setPen(QColor(209, 233, 245));	// color for highlighted instrument line
			// fill line
			for (uint j = 1; j < m_nGridHeight; j++) {
				p.drawLine( 0, y + j, (100 + m_nEditorWidth), y + j);
			}
		}
		else {
			// fill line
			for (uint j = 1; j < m_nGridHeight; j++) {
				if (i % 2 != 0) {
					p.setPen(QColor(240, 240, 255));
					p.drawLine( 0, y + j, (100 + m_nEditorWidth), y + j);
				}
			}
		}
	}

	// vertical lines
	QColor res_1(0, 0, 0);
	QColor res_2(100, 100, 100);
	QColor res_3(150, 150, 150);
	QColor res_4(200, 200, 200);
	QColor res_5(230, 230, 230);

	p.setPen(res_1);
	p.drawLine(0, 0, 0, m_nEditorHeight);

	// horizontal lines
	for (uint i = 0; i < MAX_INSTRUMENTS; i++) {
		uint y = m_nGridHeight * i;
		p.setPen(QColor(0, 0, 0));
		p.drawLine( 0, y, (100 + m_nEditorWidth), y);

		p.setPen(QColor(0, 0, 0));
		uint text_y = (m_nGridHeight * MAX_INSTRUMENTS) - (i * m_nGridHeight);
		string trackName = "";

		Instrument *instr = instrList->get(i);
		trackName = instr->getName();
		p.drawText( 5, text_y - 1, trackName.c_str());
	}
//	p.drawLine( 0, m_nEditorHeight - 1, m_nEditorWidth, m_nEditorHeight -1);

	p.setPen(QColor(0, 0, 0));
	p.drawLine( 0, m_nGridHeight * MAX_INSTRUMENTS, m_nEditorWidth, m_nGridHeight * MAX_INSTRUMENTS);

	p.drawLine(m_nEditorWidth - 1 , 0, m_nEditorWidth - 1 , m_nEditorHeight);

}




/**
 * This method is called from another thread (audio engine)
 */
void PatternEditorInstrumentList::patternChanged() {
	H2TextEvent *ev = new H2TextEvent("patternChanged");
	QApplication::postEvent(this, ev);

}



/**
 *
 */
void PatternEditorInstrumentList::customEvent( QCustomEvent *ev ) {
	if ( ev->type() != H2_TEXT_EVENT ) {	// Must be a H2TextEvent
		return;
	}
	string message = ((H2TextEvent*)ev)->getText();

	if (message == "patternChanged") {
		updateEditor();
	}

}




/**
 *
 */
void PatternEditorInstrumentList::updateEditor() {
	if(!isVisible()) {
		return;
	}

	Hydrogen* engine = Hydrogen::getInstance();

	// check engine state
	int state = engine->getState();
	if ( (state != READY) && (state != PLAYING) ) {
		cerr << "TODO: skip pattern editor update (state shoud be READY or PLAYING" << endl;
		return;
	}

	createBackground(&background);
	changed = true;
	update();
}






/**
 *
 */
void PatternEditorInstrumentList::mousePressEvent(QMouseEvent *ev) {
	int row = MAX_INSTRUMENTS - 1 - (ev->y()  / (int)m_nGridHeight);
	if (row >= MAX_INSTRUMENTS) {
		return;
	}

	m_pPatternEditor->setSelectedInstrument( row );
	updateEditor();
	m_pPatternEditor->updateEditor();

	if (ev->button() == RightButton ) {
		functionPopup->popup( QPoint( ev->globalX(), ev->globalY() ) );
	}
	return;

}




/**
 *
 */
void PatternEditorInstrumentList::clearNotes() {
	Hydrogen *engine = (Hydrogen::getInstance());
	engine->lockEngine();	// lock the audio engine

	Pattern* currentPattern = engine->getCurrentPattern();
	SequenceList *sequenceList = currentPattern->getSequenceList();
	Sequence *seq = sequenceList->get( m_pPatternEditor->getSelectedInstrument() );

	for (uint i = 0; i < MAX_NOTES; i++) {
		if (seq->noteList[i] != NULL) {
			Note *note = seq->noteList[i];
			delete note;
			seq->noteList[i] = NULL;
		}
	}
	engine->unlockEngine();	// unlock the audio engine

	changed = true;
	update();
	m_pPatternEditor->update();
}




/**
 *
 */
void PatternEditorInstrumentList::fillNotes() {
	Hydrogen *engine = (Hydrogen::getInstance());
	engine->lockEngine();	// lock the audio engine

	Pattern* currentPattern = engine->getCurrentPattern();
	SequenceList *sequenceList = currentPattern->getSequenceList();
	Sequence *seq = sequenceList->get( m_pPatternEditor->getSelectedInstrument() );
	Song *song = ( Hydrogen::getInstance() )->getSong();
	Instrument *instrRef = (song->getInstrumentList())->get( m_pPatternEditor->getSelectedInstrument() );
	float velocity = 0.8;
	float pan_L = 1.0;
	float pan_R = 1.0;
	int m_nResolution = 1;
	for ( uint i = 0; i < MAX_NOTES; i += m_nResolution ) {

		if (seq->noteList[i] == NULL ) {
			// create the new note
			Note *note = new Note( i, velocity, pan_L, pan_R );
			note->setInstrument(instrRef);
			seq->noteList[i] = note;
		}
	}

	engine->unlockEngine();	// unlock the audio engine

	changed = true;
	update();
	m_pPatternEditor->update();

}
