/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: PatternEditor.h,v 1.24 2003/09/25 16:17:28 comix Exp $
 *
 */


#ifndef PATTERN_EDITOR_H
#define PATTERN_EDITOR_H

#include "qwidget.h"
#include "qtimer.h"
#include "qpixmap.h"
#include "qpainter.h"
#include "qapplication.h"
#include "qpopupmenu.h"


#include <hydrogen/Song.h>
#include <hydrogen/Hydrogen.h>
#include <hydrogen/EngineListener.h>
#include <hydrogen/AlsaMidiDriver.h>

#include "Globals.h"
#include "config.h"
#include "HydrogenApp.h"

class PatternEditorInstrumentList;

/**
 * Pattern editor
 */
class PatternEditor : public QWidget, public EngineListener {
	Q_OBJECT

	public:
		PatternEditor(QWidget* parent);
		~PatternEditor();

		void mousePressEvent(QMouseEvent *ev);
		void paintEvent(QPaintEvent *ev);
		void setResolution(uint res, bool bUseTriplets);

		void showEvent ( QShowEvent *ev );
		void hideEvent ( QHideEvent *ev );

		int getSelectedInstrument() {	return selectedInstrument;	}
		void setSelectedInstrument(int nInstr);

		void setPatternEditorInstrumentList( PatternEditorInstrumentList* editor ) {	m_pPatternEditorInstrumentList = editor;	}

	public slots:
		void updateEditor();

	private:
		uint m_nEditorWidth;
		uint m_nEditorHeight;

		uint m_nGridWidth;
		uint m_nGridHeight;

		QPixmap temp;
		QPixmap background;
		QPixmap gridAndNotes;

		bool changed;
		bool notesChanged;

		uint m_nResolution;
		bool m_bUseTriplets;

		Pattern *currentPattern;

		uint selectedInstrument;

		PatternEditorInstrumentList *m_pPatternEditorInstrumentList;

		void drawNote(Note* note, uint nSequence, QPixmap *pixmap);
		void drawPattern(QPixmap *pixmap);
		void createBackground(QPixmap *pixmap);

		virtual void patternChanged();
		void customEvent( QCustomEvent *ev );

};





/**
 *
 */
class PatternEditorRuler : public QWidget
{
	Q_OBJECT

	public:
		PatternEditorRuler(QWidget* parent);
		~PatternEditorRuler();

		void paintEvent(QPaintEvent *ev);
		void updateStart(bool start);

		void showEvent ( QShowEvent *ev );
		void hideEvent ( QHideEvent *ev );

	public slots:
		void updateEditor();

	private:
		uint m_nRulerWidth;
		uint m_nRulerHeight;
		uint m_nGridWidth;
		QPixmap background;
		QPixmap tickPosition;
		QPixmap temp;
		QTimer *timer;
		uint nTicks;
		bool changed;
};



/**
 *
 */
class PatternEditorInstrumentList : public QWidget, public EngineListener {
	Q_OBJECT

	public:
		PatternEditorInstrumentList( QWidget *parent, PatternEditor *pPatternEditor );
		~PatternEditorInstrumentList();


	public slots:
		void updateEditor();
		void clearNotes();
		void fillNotes();


	private:
		PatternEditor *m_pPatternEditor;
		static const uint m_nGridHeight = 12;
		uint m_nEditorWidth;
		uint m_nEditorHeight;
		bool changed;
		QPixmap background;
		QPixmap temp;

		QPixmap genericIcon;
		QPopupMenu *functionPopup;


		virtual void patternChanged();
		void customEvent( QCustomEvent *ev );
		void paintEvent(QPaintEvent *ev);
		void createBackground(QPixmap *pixmap);
		void mousePressEvent(QMouseEvent *ev);
};


#endif
