/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: PlayerControl.cpp,v 1.39 2003/09/17 16:23:25 comix Exp $
 *
 */


#include "PlayerControl.h"

/**
 * Constructor
 */
PlayerControl::PlayerControl(QWidget *parent) : QWidget(parent) {
	setMinimumSize( width, height );
	setMaximumSize( width, height );
	resize( width, height );

	string background_path = string( IMG_PATH ) + string( "/img/playerControlPanel/background.png" );
	bool ok = background.load( background_path.c_str() );
	if( ok == false ){
		qWarning( "PlayerControl: Error loading pixmap" );
	}
	setBackgroundPixmap( background );

	// Play button
	string play_on_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_play_on.png" );
	string play_off_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_play_off.png" );
	string play_over_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_play_over.png" );
	playBtn = new ToggleButton(this, QSize(20, 20), play_on_path, play_off_path, play_over_path);
	playBtn->move(5, 5);
	playBtn->setPressed(false);
	QToolTip::add( playBtn, trUtf8("Play") );

	connect(playBtn, SIGNAL(clicked(Button*)), this, SLOT(playBtnClicked(Button*)));

	// Stop button
	string stop_on_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_stop_on.png" );
	string stop_off_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_stop_off.png" );
	string stop_over_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_stop_over.png" );
	stopBtn = new ToggleButton(this, QSize(20, 20), stop_on_path, stop_off_path, stop_over_path);
	stopBtn->move(30, 5);
	stopBtn->setPressed(true);
	QToolTip::add( stopBtn, trUtf8("Stop") );

	connect(stopBtn, SIGNAL(clicked(Button*)), this, SLOT(stopBtnClicked(Button*)));


	// Song mode button
	string songMode_on_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_song_on.png" );
	string songMode_off_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_song_off.png" );
	string songMode_over_path = string( IMG_PATH) + string( "/img/playerControlPanel/btn_song_over.png" );
	songModeBtn = new ToggleButton(this, QSize(20, 20), songMode_on_path, songMode_off_path, songMode_over_path);
	songModeBtn->move(95, 5);
	songModeBtn->setPressed(false);
	QToolTip::add( songModeBtn, trUtf8("Song Mode") );

	connect(songModeBtn, SIGNAL(clicked(Button*)), this, SLOT(songModeBtnClicked(Button*)));

	// Live mode button
	string liveMode_on_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_live_on.png" );
	string liveMode_off_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_live_off.png" );
	string liveMode_over_path = string( IMG_PATH ) + string( "/img/playerControlPanel/btn_live_over.png" );
	liveModeBtn = new ToggleButton(this, QSize(20, 20), liveMode_on_path, liveMode_off_path, liveMode_over_path);
	liveModeBtn->move(70, 5);
	liveModeBtn->setPressed(true);
	QToolTip::add( liveModeBtn, trUtf8("Live Mode") );

	connect(liveModeBtn, SIGNAL(clicked(Button*)), this, SLOT(liveModeBtnClicked(Button*)));


	// BPM
	bpmSpinBox = new QSpinBox( this, "bpmSpinBox" );
	bpmSpinBox->setMaxValue( 300 );
	bpmSpinBox->setMinValue( 40 );
	bpmSpinBox->setLineStep( 1 );
	bpmSpinBox->setValue( 120 );
	bpmSpinBox->move( 135, 5 );
	bpmSpinBox->resize( 45, 20 );
	bpmSpinBox->setEnabled( true );
	QToolTip::add( bpmSpinBox, trUtf8("BPM") );
	connect(bpmSpinBox, SIGNAL(valueChanged(int)), this, SLOT(bpmChanged()));
	engine = Hydrogen::getInstance();

	// CPU load widget
	cpuLoadWidget = new CpuLoadWidget( this );
	cpuLoadWidget->move( 200, 5 );

	// Midi Activity widget
	midiActivityWidget = new MidiActivityWidget( this );
	midiActivityWidget->move( 310, 5 );


	QTimer *timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(updatePlayerControl()));

	timer->start(200);	// update player control at 5 fps
}




/**
 * Destructor
 */
PlayerControl::~PlayerControl() {
}





/**
 * Start audio engine
 */
void PlayerControl::playBtnClicked(Button* ref) {
	if (ref->isPressed()) {
		stopBtn->setPressed(false);
		// Start audio engine
		engine->start();
		(HydrogenApp::getInstance())->setStatusBarMessage(trUtf8("Playing.").latin1(), 5000);
	}
	else {
		playBtn->setPressed(true);
		stopBtn->setPressed(false);
	}
}




/**
 * Stop audio engine
 */
void PlayerControl::stopBtnClicked(Button* ref) {
	if (ref->isPressed()) {
		playBtn->setPressed(false);
		engine->stop();
		bpmSpinBox->setEnabled(true);
		(HydrogenApp::getInstance())->setStatusBarMessage(trUtf8("Stopped.").latin1(), 5000);
	}
	else {
		stopBtn->setPressed(true);
		playBtn->setPressed(false);
		bpmSpinBox->setEnabled(true);
	}

}




/**
 * Set Song mode
 */
void PlayerControl::songModeBtnClicked(Button* ref) {
	(Hydrogen::getInstance())->getSong()->setMode( SONG_MODE );
	songModeBtn->setPressed(true);
	liveModeBtn->setPressed(false);
	(HydrogenApp::getInstance())->setStatusBarMessage(trUtf8("Song mode selected.").latin1(), 5000);
}




/**
 * Set Live mode
 */
void PlayerControl::liveModeBtnClicked(Button* ref) {
	(Hydrogen::getInstance())->getSong()->setMode( LIVE_MODE );
	songModeBtn->setPressed(false);
	liveModeBtn->setPressed(true);
	(HydrogenApp::getInstance())->setStatusBarMessage(trUtf8("Live mode selected.").latin1(), 5000);

}




/**
 *
 */
void PlayerControl::updatePlayerControl() {
	int state = (Hydrogen::getInstance())->getState();
	if (state == PLAYING ) {
		playBtn->setPressed(true);
		stopBtn->setPressed(false);
	}
	else {
		playBtn->setPressed(false);
		stopBtn->setPressed(true);
	}

	Song *song = (Hydrogen::getInstance())->getSong();
	int bpm = song->getBpm();
	bpmSpinBox->setValue(bpm);

	if ( song->getMode() == LIVE_MODE ) {
		liveModeBtn->setPressed( true );
		songModeBtn->setPressed( false );
	}
	else {
		liveModeBtn->setPressed( false );
		songModeBtn->setPressed( true );
	}



}





/**
 *
 */
void PlayerControl::bpmChanged() {
	int newBpmValue = bpmSpinBox->value();
//	cout << "bpmChanged " << newBpmValue << endl;
	Song *song = (Hydrogen::getInstance())->getSong();
	song->setBpm(newBpmValue);
}



