/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: SongEditor.h,v 1.21 2003/06/24 15:18:03 comix Exp $
 *
 */

#ifndef SONG_EDITOR_H
#define SONG_EDITOR_H

#include "qwidget.h"
#include "qpixmap.h"
#include "qscrollview.h"
#include "qvbox.h"
#include "qpopupmenu.h"

#include <string>
using std::string;

#include "config.h"
#include <hydrogen/Song.h>
#include <hydrogen/Hydrogen.h>
#include <hydrogen/EngineListener.h>

#include "HydrogenApp.h"
#include "Button.h"
#include "PatternPropertiesDialog.h"
#include "SongPropertiesDialog.h"

class SongEditor;
class SongEditorPatternList;
class SongEditorPositionRuler;

/**
 * Song Editor panel
 */
class SongEditorPanel : public QWidget, public EngineListener {
	Q_OBJECT

	public:
		SongEditorPanel( QWidget *parent );
		~SongEditorPanel();

		void updateAll();
	public slots:
		void contentsMove( int x, int y );
		void newPatBtnClicked( Button* );
		void songBtnClicked( Button* );
		void upBtnClicked( Button* );
		void downBtnClicked( Button* );
		void songLoopBtnClicked(Button* ref);

		void songPopup_songProperties();
		void songPopup_clearSequence();
		void songPopup_deleteAllPatterns();

	private:
		static const uint initialWidth = 600;
		static const uint initialHeight = 250;

		QScrollView* sv1;
		QScrollView* sv2;
		QScrollView* sv3;

		SongEditor *songEditor;
		SongEditorPatternList *songEditorPatternList;
		SongEditorPositionRuler *songEditorPositionRuler;

		Button *upBtn;
		Button *downBtn;

		Button *songBtn;
		QPopupMenu *songPopup;

		ToggleButton *songLoopBtn;

		virtual void patternChanged();
		void customEvent( QCustomEvent *ev );

		void resizeEvent ( QResizeEvent *ev );

};




/**
 * Song editor
 */
class SongEditor : public QWidget, public EngineListener {
	Q_OBJECT

	public:
		SongEditor( QWidget *parent );
		~SongEditor();

		void updateEditor();
		void createBackground();

	private:
		static const uint gridWidth = 16;
		static const uint gridHeight = 12;
		static const uint maxPatternSequence = 300;
		static const uint initialWidth = maxPatternSequence * gridWidth;
		static const uint initialHeight = 10;
		bool changed;
		bool sequenceChanged;
		QPixmap temp;
		QPixmap background;
		QPixmap sequence;

		void mousePressEvent(QMouseEvent *ev);
		void paintEvent(QPaintEvent *ev);
		void drawSequence();
		void drawPattern( int pos, int number );

};




/**
 * Song editor pattern list
 */
class SongEditorPatternList : public QWidget, public EngineListener {
	Q_OBJECT

	public:
		SongEditorPatternList( QWidget *parent );
		~SongEditorPatternList();

		void updateEditor();
		void createBackground();
		int getSelectedPatternPos() {	return nSelectedPattern;	}

	public slots:
		void patternPopup_edit();
		void patternPopup_properties();
		void patternPopup_delete();
		void patternPopup_copy();


	private:
		static const uint gridHeight = 12;
		static const uint width = 150;
		static const uint initialHeight = 10;
		bool changed;
		QPixmap temp;
		QPixmap background;
		int nSelectedPattern;

		QPopupMenu *patternPopup;

		void mousePressEvent( QMouseEvent *ev );
		void mouseDoubleClickEvent( QMouseEvent *ev );
		void paintEvent( QPaintEvent *ev );

};



/**
 *
 */
class SongEditorPositionRuler : public QWidget {
	Q_OBJECT

	public:
		SongEditorPositionRuler( QWidget *parent );
		~SongEditorPositionRuler();

		void updateRuler();
		void createBackground();

	private:
		static const uint gridWidth = 16;
		static const uint maxPatternSequence = 300;
		static const uint initialWidth = maxPatternSequence * gridWidth;
		static const uint height = 15;

		bool changed;
		QPixmap temp;
		QPixmap background;
		QPixmap tickPosition;

		void mousePressEvent( QMouseEvent *ev );
		void paintEvent( QPaintEvent *ev );

};

#endif
